## ----import ECDC data---------------------------------------------------------
ecdc_long <- read.csv(
    file = "data/ECDC_surveillance_data_IMD.csv.gz",  # R can read compressed files
    na.strings = "-")  # always important to know how NA's are encoded!
head(ecdc_long, 10)

## exclude aggregate counts
ecdc_long <- subset(ecdc_long, !startsWith(RegionName, "EU"))

## reshape from long to wide format of multivariate time series
ecdc <- reshape(ecdc_long[c("Time", "RegionCode", "NumValue")],
                direction = "wide", idvar = "Time", timevar = "RegionCode")
names(ecdc) <- sub("NumValue.", "", names(ecdc), fixed = TRUE)
row.names(ecdc) <- ecdc$Time; ecdc$Time <- NULL
head(ecdc)

## restrict to countries with complete data (KISS)
ecdc <- ecdc[, colSums(is.na(ecdc)) == 0]
tail(ecdc)


## ----import population data---------------------------------------------------
popdata <- read.csv("data/Eurostat_population_2018.csv")
head(popdata, 3)
stopifnot(colnames(ecdc) %in% popdata$geo)
pop <- setNames(popdata$OBS_VALUE, popdata$geo)[colnames(ecdc)]
pop




## ----load map-----------------------------------------------------------------
load("data/map.RData", verbose = TRUE)




## ----plot map-----------------------------------------------------------------
library("sp")
par(mar = c(0,0,0,0)) # no margin
plot(map)


## ----start year/index---------------------------------------------------------
(start <- as.numeric(strsplit(rownames(ecdc), "-")[[1]]))


## ----create sts, results = "hide"---------------------------------------------
library("surveillance")

## See help("sts") and learn about the function arguments:
str(sts)
## Complete the following expression:
## IMD <- sts(....)


## ----solution = TRUE----------------------------------------------------------
IMD <- sts(ecdc, start = start, frequency = 12, # monthly data
           population = pop, map = map)


## ----print the created sts object---------------------------------------------
IMD


## ----basic methods for sts, include = FALSE-----------------------------------
##
## help(sts)  # Section "Methods"
##


## ----solution = TRUE----------------------------------------------------------
dim(IMD)  # months x countries
colnames(IMD)

epoch(IMD, as.Date = TRUE)
year(IMD)
epochInYear(IMD)

observed(IMD) |> head()
population(IMD) |> head()  # same dim() as 'observed' (could vary over time)

## note that "sts" objects have some more slots (used by monitoring algorithms)
showClass("sts")


## ----solution = TRUE----------------------------------------------------------
## subsetting
IMD[1:3, "UK"] # a very short, univariate "sts"
tail(IMD, 12)  # only the last 12 months
IMD[year(IMD) > 2011,]


## ----solution = TRUE----------------------------------------------------------
IMD6 <- IMD[, grep("^[DFN]", colnames(IMD))]
IMD6


## ----solution = TRUE----------------------------------------------------------
## collapse the multivariate time series in either dimension
## help(aggregate.sts)
aggregate(IMD6, by = "time")  # summing over time
aggregate(IMD6, by = "unit")  # overall time series

## convert to a data frame (omitting non-temporal slots)
as.data.frame(IMD6)              |> head(3)  # wide format
as.data.frame(IMD6, tidy = TRUE) |> head(3)  # long format
IMD6DF <- tidy.sts(IMD6)  # the same
## the long data format is convenient for, e.g., custom 'lattice' plots
lattice::xyplot(observed ~ date | unit, data = IMD6DF, type = "h", as.table = TRUE)

## convert to R's basic "ts" (and use its methods)
as.ts(IMD6[1:24,])
(nlts <- as.ts(IMD6[,"NL"]))
plot(nlts)
monthplot(nlts)

## if you have package 'xts', this also works:
xts::as.xts(IMD6) |> # then using the plot-method from 'xts'
    plot(multi.panel = TRUE, yaxis.same = FALSE)


## ----plot sts, include = FALSE------------------------------------------------
##
## help(plot.sts)
##


## ----solution = TRUE----------------------------------------------------------
## standard plot
plot(IMD6)

## some useful arguments, see ?stsplot_time:
plot(IMD6, same.scale = FALSE)
plot(IMD6, epochsAsDate = TRUE)  # alternative time axis setup
plot(IMD6, units = c("NL","FR")) # select some time series


## ----solution = TRUE----------------------------------------------------------
autoplot.sts(IMD6)
autoplot.sts(IMD6, scales = "free")
autoplot.sts(IMD6, population = 100000)
autoplot.sts(IMD6, population = 100000, as.one = TRUE)

## or manually based on the tidy.sts() data frame created above:
library("ggplot2")
ggplot(IMD6DF, aes(x = date, y = observed)) + geom_col() + facet_wrap(~ unit)


## ----solution = TRUE----------------------------------------------------------
plot(IMD6, type = observed ~ time)

## the same:
IMD1 <- aggregate(IMD6, by = "unit")
plot(IMD1)
autoplot.sts(IMD1)


## ----solution = TRUE----------------------------------------------------------
plot(IMD6, type = observed ~ unit)

## only for the year 2018
plot(IMD6, type = observed ~ unit,
     tps = which(year(IMD6) == 2018),
     total.args = list()) # annotate with total number

## cumulative incidence (per 100'000 inhabitants),
## using some of the graphical options
plot(IMD6, type = observed ~ unit, population = 100000,
     gpar.missing = list(lty = 1, col = 8),
     col = "white", lwd = 2, # thick white borders
     sub = "cumulative incidence (per 100'000 inhabitants)")

