## ----data---------------------------------------------------------------------
library("surveillance")
load("data/data_BE.RData", verbose = TRUE)


## ----aggregate----------------------------------------------------------------
noro <- aggregate(noroBE, by = "unit")
noro


## ----plot, solution = TRUE----------------------------------------------------
plot(noro)

## alternative visualizations (and there are probably more)
plot(noro, col = 8, epochsAsDate = TRUE)  # alternative colour & time axis setup
plot(as.ts(noro))
ggplot2::autoplot(noro)
plot(xts::as.xts(noro))

## there is a clear yearly seasonality, see also
## the basic "ts" monthplot()
as.ts(noro) |> monthplot(lwd.base = 3)
## => (unfortunate) peak around Christmas, minimum around calendar week 32

## or use, e.g., lattice::xyplot() with the "tidy" dataset
lattice::xyplot(observed ~ epochInYear, data = tidy.sts(noro),
                groups = year, type = "l", xlab = "calendar week")


## ----sine-cosine terms--------------------------------------------------------
f1 <- addSeason2formula(~ 1, S = 1, period = 52)
f1


## ----hhh4 call, eval = FALSE--------------------------------------------------
## fit <- hhh4(stsObj, control)


## ----control argument for an intercept-only model, eval = FALSE---------------
## control = list(
##     ar  = list(f = ~ 1),
##     end = list(f = ~ 1),
##     family = "NegBin1"
## )


## ----update, eval = FALSE-----------------------------------------------------
## fit2 <- update(fit, ar = list(f = f1))


## ----solution = TRUE----------------------------------------------------------
## try an endemic-only model with one harmonic
fit_endemic <- hhh4(noro, list(end = list(f = f1), family = "NegBin1"))

## try the intercept-only endemic-epidemic model
fit00 <- hhh4(noro, list(ar = list(f = ~1), end = list(f = ~1), family = "NegBin1"))

## try various options with seasonality
fit01 <- update(fit00, S = list(ar = 0, end = 1))  # constant lambda
fit10 <- update(fit00, S = list(ar = 1, end = 0))  # constant nu
fit11 <- update(fit00, S = list(ar = 1, end = 1))  # both seasonal

AIC(fit_endemic, fit00, fit01, fit10, fit11)
## 'fit11' wins, but the more parsimonious 'fit01' is pretty close


## ----solution = TRUE----------------------------------------------------------
## show the coefficients with standard errors:
summary(fit01)

## transform sine-cosine coefficients to amplitude-shift parameters,
## exp-transform the intercepts to get to the response scale,
## and show the dominant eigenvalue ("=" epidemic proportion)
summary(fit01, amplitudeShift = TRUE, idx2Exp = TRUE, maxEV = TRUE)
## epidemic proportion of disease incidence is estimated to be 62%
## the rest isn't necessarily non-epidemic, but imported/unexplained

## show the fitted model components
plot(fit01) -> .components

## calculate the "empirical" epidemic proportion
str(.components)
sum(.components$overall[,"epidemic"]) / sum(.components$overall[,"mean"])

## show the estimated seasonal (multiplicative) effect
plot(fit01, type = "season")

## PIT plot to check model calibration
opar <- par(mfrow = c(1,2))
pit(fit01); title(main = "constant lambda")
pit(fit11); title(main = "seasonal lambda")
par(opar)
## no clear preference

## check residual correlation
opar <- par(mfrow = c(1,3))
acf(residuals(fit_endemic), main = "endemic-only") # strong residual correlation
acf(residuals(fit01), main = "constant lambda")
acf(residuals(fit11), main = "seasonal lambda")
par(opar)
## no clear preference either


## ----solution = TRUE----------------------------------------------------------
## compute "rolling" weekly forecasts starting from 2016
startidx <- match(2016, year(noro))
osa01 <- oneStepAhead(fit01, startidx, verbose = FALSE)
plot(osa01)
head(quantile(osa01)) # median and prediction intervals
pit(osa01) # model tends to underpredict

## forecast from fit11 and compare mean scores() (where smaller is better)
osa11 <- oneStepAhead(fit11, startidx, verbose = FALSE)
rbind("constant lambda" = colMeans(scores(osa01)),
      "seasonal lambda" = colMeans(scores(osa11)))
## LS/RPS favour seasonal lambda, DSS favours constant lambda

